###################
#
# Efficiency.R
#
# Author: Gael Yvert
#
# Reads StepOne plus data
# And return Amplification Efficiencies
#
# Method is as described in Pfaffl N.A.R 2001 29(9).
#
#####################

# NOTES:
# Field Quantity must contain the dilution informations
# Field Task must be set to STANDARD for all dilution samples

###################################
# read infos from Config.txt      #
###################################
cfg <- read.table("Config.txt", sep = "\t" )
unexpected <- (cfg[1,1] !=  "expname");
if( sum(unexpected) != 0)
  stop("unexpected field name(s): ", cfg[1,1]);
if(dim(cfg)[2]!=2)
  stop("configuration file should have two columns")

exp.name = cfg[1,2];

###########################
# re-format RESULTS file  #
###########################
resfile1 = paste(exp.name, "_results.txt", sep = "");
resfile2 = paste(exp.name, "_results.dat", sep = "");
cmd <- paste("awk 'BEGIN{start=0; stop=0;counter = -1;} { s1 = substr($1, 2, 7)} s1 ~ /Results/ {start = 1; counter =0;} {if (start && !stop) {print $0; if (NF ==1 && counter>0) stop = 1}; counter++;  }' ", resfile1, " > ", resfile2, sep = "");
system(cmd);

###########################
# read results data       #
###########################
dat <- read.table(resfile2, skip = 1, header = TRUE, sep = "\t", na.strings = c("", "Undetermined"))
x <- dat[!is.na(dat$Target.Name) & dat$Task == "STANDARD",c(1,3,4,7,8,10,19,20,21)]
x <- unique(x) #in case there are duplicates

###########################
# verify genes names      #
###########################
if ( sum(!(levels(cfg$V1) %in% c("expname", levels(x$Target.Name)))) != 0 )
  stop ("Gene names in configuration file differ from those of results file")

###########################
# compute efficiencies    #
###########################
amplicons <- as.character(levels(x$Target.Name))
E <- vector("numeric", length = length(amplicons))
i = 1;
for (amp in levels(x$Target.Name)){
	#see in config which dilutions should be ignored
	excluded.quantities = as.numeric(as.character(cfg$V2[cfg$V1 == amp]))
	#perform
	sx <- subset(x, Target.Name == amp & Quantity & !(Quantity %in% excluded.quantities))
	model <- lm(sx[,5] ~log10(sx$Quantity))
	#store a plot of data and model
	pdf(file = paste(exp.name, "_", amp, "_std.pdf", sep = ""))
	plot(sx[,5] ~log10(sx$Quantity), ylab = "Ct", xlab = "Log10(Quantity)", main = amp);
	legend("topright", c("excluded: ", excluded.quantities));
	abline(model)
	dev.off()
	E[i] <- 10^(-1/model$coefficients[2])
	i = i+1;
}

outp <- data.frame(Amplicon = amplicons, Efficiency = E)

# write to a file
write.table(outp, file = paste(exp.name, "_efficiencies.txt", sep = ""), sep = "\t", quote = FALSE, row.names = FALSE);
