###################
#
# ExpRatio.R
#
# Author: Gael Yvert
#
# Reads StepOne plus data
# And return Expresion ratio
# of gene of interest$
# with respect to control gene (such as ACT1)
#
# Method is as described in Pfaffl N.A.R 2001 29(9).
#
#####################

# NOTES: 
# 1) Efficiencies (Based on prior linear dilutions experiments)
# should be entered manually in the _results.txt file
# 2) Informations should be supplied in configuration file "Config.txt"


###################################
# read parameters from Config.txt #
###################################
cfg <- read.table("Config.txt", sep = "\t" )
unexpected <- (cfg[1:3,1] !=  c("expname", "mygene", "housegene"));
if( sum(unexpected) != 0)
  stop("unexpected field name(s): ", cfg[1:3,1][unexpected]);
if(dim(cfg)[2]!=2)
  stop("configuration file should have two columns")

exp.name = cfg[1,2];
mygene = as.character(cfg[2,2]);
housegene = as.character(cfg[3,2]);

###########################
# re-format RESULTS file  #
###########################
resfile1 = paste(exp.name, "_results.txt", sep = "");
resfile2 = paste(exp.name, "_results.dat", sep = "");
cmd <- paste("awk 'BEGIN{start=0; stop=0;counter = -1;} { s1 = substr($1, 2, 7)} s1 ~ /Results/ {start = 1; counter =0;} {if (start && !stop) {print $0; if (NF ==1 && counter>0) stop = 1}; counter++;  }' ", resfile1, " > ", resfile2, sep = "");
system(cmd);

###########################
# read results data       #
###########################
dat <- read.table(resfile2, skip = 1, header = TRUE, sep = "\t", na.strings = c("", "Undetermined"))
dat <- subset(dat, Task != "STANDARD")
levels(dat[,2])[table(dat[,2]) == 0] <- NA
x <- dat[!is.na(dat$Sample.Name),c(2,3,8,9,17)]
x <- unique(x)

###########################
# verify genes names      #
###########################
if ( sum(!(levels(x$Target.Name) %in% c(mygene, housegene))) != 0 )
  stop ("Gene names in results file differ from ", mygene, " or ", housegene)

############################
# verify that sample names #
# are consistent with      #
# config file              #
############################
check <- sort(levels(x$Sample.Name)) != sort(as.character(cfg$V1[4:dim(cfg)[1]]))
if (sum( check ) != 0)
  stop("Inconsistency between smaple names of config file and results file\n", "in config: ", sort(levels(x$Sample.Name))[check], "\nin results:", sort(as.character(cfg$V1[4:dim(cfg)[1]]))[check], "\n");

######################################
# verify small SD in replicates      #
######################################

#todo

#############################
# compute expression ratios #
#############################

#prepare output
smp <- vector("character", length = nlevels(x$Sample.Name))
refsmp <- vector("character", length = nlevels(x$Sample.Name))
expratio <- vector("numeric", length = nlevels(x$Sample.Name))
#compute expratio:
jct = 3; #column of x containing the Ct
i = 1;
for (s in levels(x$Sample.Name)){
	this.sample <- ( x$Sample.Name == s )
	ref.sample = as.character(cfg[cfg$V1 == s,2])
	refsmp[i] = ref.sample
	#grab values of the sample
	E.mygene = x$Efficiency[this.sample & x$Target.Name == mygene];
	E.housegene = x$Efficiency[this.sample & x$Target.Name == housegene];
	ct.mygene.sample = x[this.sample & x$Target.Name == mygene,jct];
	ct.housegene.sample = x[this.sample & x$Target.Name == housegene,jct];
	#grab values of the corresponding reference sample (as defined in config file)
	ct.mygene.ref    <- x[x$Sample.Name == ref.sample & x$Target.Name == mygene , jct];
	ct.housegene.ref <- x[x$Sample.Name == ref.sample & x$Target.Name == housegene , jct];
	#compute ratio
	num = E.mygene^(ct.mygene.ref - ct.mygene.sample);
	denom = E.housegene^(ct.housegene.ref - ct.housegene.sample);
	expratio[i] = num/denom;
	smp[i] = as.character(s);
	i = i + 1;
	#cat(s, " ", ct.mygene.sample, " ", ref.sample, " ", ct.mygene.ref, " ", num, " ", denom, " ", expratio[i-1], "\n");
}
outp <- data.frame(Sample = smp, RefSample = refsmp, ExpRatio = expratio)
names(outp)[3] <- paste(mygene, "/", housegene, "_relative_to_RefSample", sep = "")

# write to a file
write.table(outp, file = paste(exp.name, "_expratio.txt", sep = ""), sep = "\t", quote = FALSE, row.names = FALSE);